import pytest
from pathlib import Path
import glob
import pytest
from chess_insight.game import Game
from chess_insight.player import Player
from stockfish import Stockfish
from datetime import datetime
import os

test_data_path = Path(__file__).parent / "test_data"


class TestGame:
    def test_FR3_game_properties(self):
        pgn_string = Path(test_data_path / "drnykterstein_lichess_0.pgn").read_text()
        game = Game(pgn_string, "DrNykterstein")

        assert game.host == "lichess.org"
        assert game.username == "DrNykterstein"
        assert game.time_class == "blitz"
        assert game.opening == "Nimzo-Larsen Attack: Classical Variation"
        assert game.url == "https://lichess.org/kx7aGZB0"
        assert isinstance(game.player, Player)
        assert isinstance(game.opponent, Player)
        assert game.result.value == "1-0"
        assert game.player.elo == 3113
        assert game.opponent.elo == 2718
        assert game.date == datetime(2023, 7, 1, 21, 56, 43)

    @pytest.mark.parametrize(
        "pgn_path",
        glob.glob(str(test_data_path / "*.pgn")),
        ids=[
            os.path.basename(path) for path in glob.glob(str(test_data_path / "*.pgn"))
        ],
    )
    def test_FR3_game(self, pgn_path):
        usr = self._get_usr(pgn_path)
        with open(pgn_path, "r") as f:
            game = Game(f.read(), usr)
        parsed_game = game.asdict()
        assert None not in parsed_game.values()

    def _get_usr(self, pgn_path: str):
        return Path(pgn_path).stem.split("_")[0]



import pandas as pd
import os
from unittest.mock import patch, MagicMock
from chess_insight import export_games_to_csv


class TestExportGamesToCSV:
    @patch("chess_insight.pd.DataFrame")
    def test_FR5_export_games_to_csv(self, mock_df):
        mock_to_csv = MagicMock()
        mock_df.return_value.to_csv = mock_to_csv
        games = [MagicMock(), MagicMock()]
        games[0].flatten.return_value = {
            "game1_key1": "game1_value1",
            "game1_key2": "game1_value2",
        }
        games[1].flatten.return_value = {
            "game2_key1": "game2_value1",
            "game2_key2": "game2_value2",
        }
        file_name = "test_games.csv"

        result = export_games_to_csv(games, file_name)

        mock_df.assert_called_once_with(
            [
                {"game1_key1": "game1_value1", "game1_key2": "game1_value2"},
                {"game2_key1": "game2_value1", "game2_key2": "game2_value2"},
            ]
        )
        mock_to_csv.assert_called_once_with(file_name, index=False)

        if os.path.exists(file_name):
            os.remove(file_name)  # cleanup



from abc import ABC
from abc import abstractmethod
from logging import Logger
from pathlib import Path
from typing import Generator

from easy_logs import get_logger
from rich.progress import track
from rich.status import Status
from stockfish import Stockfish

from chess_insight.game import Game

logger = get_logger()


class ApiCommunicator(ABC):
    HOST: str = None

    def __init__(self, stockfish_path: Path = "stockfish.exe", depth: int = 10) -> None:
        """
        Summary:
            Abstract class for API communication. It is used to get games from chess.com, lichess, etc.
            Each subclass should implement get_games method.
        Args:
            logger (Logger): logger to log to
            depth (int, optional): depth of stockfish engine. Defaults to 10.
        """
        if not stockfish_path or not Path(stockfish_path).exists():
            logger.warning(
                f"Stockfish does not exists in given path {stockfish_path}. Therefore won't be used."
            )
            self.stockfish = None
            return
        try:
            stockfish_path = Path(stockfish_path).resolve()
            self.stockfish = Stockfish(stockfish_path.resolve(), depth=depth)
        except (AttributeError, FileNotFoundError) as err:
            raise err

    def split_pgns(self, text_pgn: str) -> list[str]:
        """
        Summary:
            Splits pgn string into list of pgn strings.
        Args:
            text_pgn (str): pgn string to split
        Returns:
            list of pgn strings
        """
        pgns = text_pgn.split("\n\n\n")
        while pgns and len(pgns[-1]) == 0:
            pgns.pop()
        return pgns

    def games_generator(
        self, username: str, count: int, time_class: str
    ) -> Generator[Game, None, None]:
        """
        Args:
            username (str): username on given portal (lichess, chess.com, etc.) to get games from
            list_of_pgns (int): number of pgn strings to compute
        returns:
            generator of Game objects, each representing a game played on chess.com
        """
        logger.info(f"Collecting games for {username} from {self.HOST}")
        list_of_pgns = self.get_pgns(username, count, time_class)
        logger.info(f"Collected {len(list_of_pgns)} games")
        logger.info(f"Collected {len(list_of_pgns)} games")
        progress = track(
            list_of_pgns,
            description=f"Analyzing games for {username}\n",
            total=len(list_of_pgns),
            transient=True,
        )
        for game in progress:
            game = Game(
                game,
                username,
                stockfish=self.stockfish,
            )
            yield game

    @abstractmethod
    def get_pgns(
        self, username: str, number_of_games: int, time_class: str
    ) -> Generator[Game, None, None]:
        """
        Args:
            username (str): username on given portal (lichess, chess.com, etc.) to get games from
            games (int): number of lastest games to get
            time_class (_type_): time class of games to get (blitz, rapid, bullet, daily)
        returns:
            generator of Game objects, each representing a game played on chess.com, licess, etc.
        """



import pytest
from unittest.mock import patch, MagicMock
from pathlib import Path
from chess_insight.api_communicator import ApiCommunicator
from chess_insight.lichess_api_communicator import LichessApiCommunicator
from chess_insight.chess_com_api_communicator import ChessComApiCommunicator


class MockApiCommunicator(ApiCommunicator):
    def get_pgns(self):
        pass


class TestApiCommunicator:
    @patch("chess_insight.api_communicator.get_logger")
    def test_init(self, mock_logger):
        mock_logger.return_value.warning = MagicMock()
        api_communicator = MockApiCommunicator("non_existent_path")
        assert api_communicator.stockfish is None

    def test_split_pgns(self):
        api_communicator = MockApiCommunicator()
        result = api_communicator.split_pgns("pgn1\n\n\npgn2\n\n\n")
        assert result == ["pgn1", "pgn2"]

    @patch("chess_insight.api_communicator.Game")
    @patch.object(MockApiCommunicator, "get_pgns")
    def test_games_generator(self, mock_get_pgns, mock_game):
        api_communicator = MockApiCommunicator()
        mock_get_pgns.return_value = ["pgn1", "pgn2"]
        mock_game.return_value = "game"
        result = list(api_communicator.games_generator("username", 2, "time_class"))
        assert result == ["game", "game"]

    @patch("chessdotcom.get_player_games_by_month_pgn")
    @patch("chessdotcom.get_player_profile")
    @patch("chess_insight.chess_com_api_communicator.get_time_class")
    @patch("chess_insight.chess_com_api_communicator.get_pgn")
    def test_FR1_get_pgns_chess_com(
        self,
        mock_get_pgn,
        mock_get_time_class,
        mock_get_player_profile,
        mock_get_player_games_by_month_pgn,
    ):
        mock_get_player_profile.return_value.json = {
            "player": {"joined": 946684800}
        }  # 2000-01-01
        mock_get_player_games_by_month_pgn.return_value.json = {
            "pgn": {"pgn": "pgn1\n\n\npgn2\n\n\n"}
        }
        mock_get_time_class.return_value = "blitz"
        mock_get_pgn.return_value.headers = {}
        api_communicator = ChessComApiCommunicator()
        result = api_communicator.get_pgns("username", 2, "blitz")
        assert result == ["pgn1", "pgn2"]

    # Add similar test for lichess.org API
    @patch.object(LichessApiCommunicator, "CLIENT")
    def test_FR1_get_pgns_lichess(self, mock_client):
        mock_export_by_player = MagicMock()
        mock_export_by_player.return_value = ["pgn1", "pgn2"]
        mock_client.games.export_by_player = mock_export_by_player
        lichess_api_communicator = LichessApiCommunicator()
        result = lichess_api_communicator.get_pgns("username", 2, "time_class")
        assert result == ["pgn1", "pgn2"]



import pytest
from unittest.mock import MagicMock
from chess_insight.player import Player
from chess_insight.utils import Color, GamePhase, Mistakes
import chess


class TestPlayer:
    def test_FR3_init(self):
        pgn = chess.pgn.Game()
        board = chess.Board()
        color = Color.WHITE
        game_phases = (1, 2, 3)
        evaluations = [{"evaluation": 1.0, "time": 2.0}]
        player = Player(pgn, board, color, game_phases, evaluations)
        assert player._pgn == pgn
        assert player._board == board
        assert player._color == color
        assert player._phases == game_phases
        assert player._evaluations == evaluations

    def test_FR3_elo(self):
        pgn = chess.pgn.Game()
        pgn.headers["WhiteElo"] = "2000"
        player = Player(
            pgn,
            chess.Board(),
            Color.WHITE,
            (1, 2, 3),
            [{"evaluation": 1.0, "time": 2.0}],
        )
        assert player.elo == 2000
